package com.muyu.source.domain;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.muyu.common.core.annotation.Excel;
import com.muyu.common.core.web.domain.BaseEntity;
import com.muyu.source.domain.req.DataSourceEditReq;
import com.muyu.source.domain.req.DataSourceQueryReq;
import com.muyu.source.domain.req.EtlDataScoreAddReq;
import com.muyu.source.pool.config.BaseConfig;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import lombok.experimental.SuperBuilder;

import java.util.function.Supplier;

/**
 * u
 *
 * @author Lenovo
 * @ Tool：IntelliJ IDEA
 * @ Author：CHX
 * @ Date：2024-08-20-10:59
 * @ Version：1.0
 * @ Description：数据源
 */


@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
@EqualsAndHashCode(callSuper = true)
@TableName(value = "data_source", autoResultMap = true)
public class DataSource extends BaseEntity {
    /**
     * 主键
     */
    @Excel(name = "主键")
    @TableId(value = "id", type = IdType.AUTO)

    private Long id;
    /**
     * 接入源名称
     */
    @Excel(name = "接入源名称")
    @TableField("name")

    private String name;
    /**
     * 数据来源系统名称
     */
    @Excel(name = "数据来源系统名称")
    @TableField("system_name")

    private String systemName;

    /**
     * 数据库类型
     */
    @Excel(name = "数据库类型")
    @TableField("data_type")

    private String dataType;

    /**
     * ip地址
     */
    @Excel(name = "ip地址")
    @TableField("ip")

    private String ip;

    /**
     * 端口号
     */
    @Excel(name = "端口号")
    @TableField("port")

    private String port;

    /**
     * 数据库名称
     */
    @Excel(name = "数据库名称")
    @TableField("database_name")

    private String databaseName;

    /**
     * 用户名
     */
    @Excel(name = "用户名")
    @TableField("user_name")

    private String userName;

    /**
     * 密码
     */
    @Excel(name = "密码")
    @TableField("password")

    private String password;

    /**
     * 数据连接参数
     */
    @Excel(name = "数据连接参数")
    @TableField("connection_param")

    private String connectionParam;

    /**
     * 是否初始化 状态 'Y'/'N'
     */
    @Excel(name = "是否初始化")
    @TableField("is_init")

    private String isInit;

    /**
     * 初始化连接数量
     */
    @Excel(name = "初始连接数量")
    @TableField("init_total")

    private Integer initTotal;

    /**
     * 最大连接数量
     */
    @Excel(name = "最大连接数量")
    @TableField("max_num")

    private Integer maxNum;

    /**
     * 最大等待时间
     */
    @Excel(name = "最大等待时间")
    @TableField("max_wait_time")

    private Integer maxWaitTime;

    /**
     * 最大等待次数
     */
    @Excel(name = "最大等待次数")
    @TableField("max_wait_size")

    private Integer maxWaitSize;
    /**
     *  驱动 com.mysql.cj.jdbc.Driver
     *  */
    @Excel(name = "驱动",defaultValue = "com.mysql.cj.jdbc.Driver")
    private String driverName;


    /**
     * 查询构造器
     */
    public static DataSource queryBuild( DataSourceQueryReq dataSourceQueryReq){
        return DataSource.builder()
                .name(dataSourceQueryReq.getName())
                .systemName(dataSourceQueryReq.getSystemName())
                .build();
    }
    /**
     * 添加
     * @param req
     * @return
     */
    public static DataSource addBuild(EtlDataScoreAddReq req){
        return DataSource.builder()
                .dataType(req.getDataType())
                .name(req.getName())
                .ip(req.getIp())
                .port(req.getPort())
                .databaseName(req.getDatabaseName())
                .userName(req.getUserName())
                .password(req.getPassword())
                .connectionParam(req.getConnectionParam())
                .isInit(req.getIsInit())
                .initTotal(req.getInitTotal())
                .maxNum(req.getMaxNum())
                .maxWaitSize(req.getMaxWaitSize())
                .maxWaitTime(req.getMaxWaitTime())
                .driverName("com.mysql.cj.jdbc.Driver")
                .build();
    }

    /**
     * 修改
     * @param req
     * @return
     */
    public static DataSource updBuild(DataSourceEditReq req, Supplier<Long> idSupplier){
        return DataSource.builder()
                .id(idSupplier.get())
                .dataType(req.getDataType())
                .name(req.getName())
                .ip(req.getIp())
                .port(req.getPort())
                .databaseName(req.getDatabaseName())
                .userName(req.getUserName())
                .password(req.getPassword())
                .connectionParam(req.getConnectionParam())
                .isInit(req.getIsInit())
                .initTotal(req.getInitTotal())
                .maxNum(req.getMaxNum())
                .maxWaitSize(req.getMaxWaitSize())
                .maxWaitTime(req.getMaxWaitTime())
                .driverName("com.mysql.cj.jdbc.Driver")
                .build();
    }

    /**
     * 获取数据库连接
     * @return
     */
    public String getUrl(){
        StringBuilder urlSb = new StringBuilder(BaseConfig.MYSQLJDBCPRO);
        //拼接ip
        urlSb.append(this.ip);
        urlSb.append(":");
        //拼接端口
        urlSb.append(this.port);
        urlSb.append("/");
        //拼接数据库
        urlSb.append(this.databaseName);
        urlSb.append("?");
        //useUnicode=true&characterEncoding=utf8&zeroDateTimeBehavior=convertToNull&useSSL=true&serverTimezone=GMT%2B8
        urlSb.append(this.connectionParam);
        return urlSb.toString();
    }

}
