package com.muyu.source.domain.resp;
import com.muyu.common.core.annotation.Excel;
import com.muyu.source.domain.DataSource;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.SuperBuilder;

/**
 * @version 1.0
 * @Author xie ya ru
 * @Date 2024/8/19 19:12
 * @注释
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@SuperBuilder
@Tag(name = "数据源输出信息",description = "查询数据源信息需要输出的字段")
public class DataSourceResp {
    /**
     * 主键
     */
    @Excel(name = "主键")
    private Long id;
    /**
     * 接入源名称
     */
    @Schema(type = "String",defaultValue = "接入源名称",description = "接入源名称")
    private String name;

    /**
     * 数据来源系统名称
     */
    @Schema(type = "String",defaultValue = "数据来源系统",description = "数据来源系统")
    private String systemName;

    /**
     * 数据库类型
     */
    @Schema(type = "String",defaultValue = "mysql等",description = "要选择的数据库类型")
    private String dataType;

    /**
     * ip地址
     */
    @Schema(type = "String",defaultValue = "ip地址",description = "ip地址,要连接的mysql的ip")
    private String ip;

    /**
     * 端口号
     */
    @Schema(type = "String",defaultValue = "3306",description = "端口号")
    private String port;

    /**
     * 数据库名称
     */
    @Schema(type = "String",defaultValue = "t_user",description = "数据库名称")
    private String databaseName;

    /**
     * 用户名
     */
    @Schema(type = "String",defaultValue = "root",description = "用户名")
    private String userName;

    /**
     * 密码
     */
    @Schema(type = "String",defaultValue = "root",description = "密码")
    private String password;

    /**
     * 数据连接参数
     */
    @Schema(type = "String",defaultValue = "useUnicode=true&characterEncoding=utf8",description = "数据连接参数")
    private String connectionParam;

    /**
     * 是否初始化
     */
    @Schema(type = "String",defaultValue = "Y",description = "是否初始化 状态 'Y'/'N'")
    private String isInit;

    /**
     * 初始连接数量
     */
    @Schema(type = "Long",defaultValue = "5",description = "初始化连接数量")
    private Integer initTotal;

    /**
     * 最大连接数量
     */
    @Schema(type = "Long",defaultValue = "10",description = "最大连接数量")
    private Integer maxNum;

    /**
     * 最大等待时间
     */
    @Schema(type = "Long",defaultValue = "10",description = "最大等待时间")
    private Integer maxWaitTime;

    /**
     * 最大等待次数
     */
    @Schema(type = "Long",defaultValue = "3",description = "最大等待次数")
    private Integer maxWaitSize;
    /**
     *  驱动 com.mysql.cj.jdbc.Driver
     *  */
    @Excel(name = "驱动",defaultValue = "com.mysql.cj.jdbc.Driver")
    private String driverName;

    public static DataSourceResp dataSourceRespBuild(DataSource dataSource){
        return DataSourceResp.builder()
                .dataType(dataSource.getDataType())
                .name(dataSource.getName())
                .ip(dataSource.getIp())
                .port(dataSource.getPort())
                .databaseName(dataSource.getDatabaseName())
                .userName(dataSource.getUserName())
                .password(dataSource.getPassword())
                .connectionParam(dataSource.getConnectionParam())
                .isInit(dataSource.getIsInit())
                .initTotal(dataSource.getInitTotal())
                .maxNum(dataSource.getMaxNum())
                .maxWaitSize(dataSource.getMaxWaitSize())
                .maxWaitTime(dataSource.getMaxWaitTime())
                .driverName("com.mysql.cj.jdbc.Driver")
                .build();
    }




































}
