package com.muyu.source.controller;

import com.dtflys.forest.springboot.annotation.ForestScannerRegister;
import com.muyu.common.core.domain.Result;
import com.muyu.source.domain.AssetAccredit;
import com.muyu.source.domain.req.AssetAccreditQueryReq;
import com.muyu.source.service.AssetAccreditService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author Lenovo
 * @ Tool：IntelliJ IDEA
 * @ Author：CHX
 * @ Date：2024-09-01-21:48
 * @ Version：1.0
 * @ Description：资产授权控制层
 */
@Log4j2
@RestController
@RequestMapping("/asset")
@Tag(name = "资产授权控制层", description = "进行资产授权管理，查看等相关操作")
public class AssetAccreditController {
    public AssetAccreditController() {
        log.info("forest扫描路径:{}", ForestScannerRegister.getBasePackages());
    }

    @Autowired
    private AssetAccreditService assetAccreditService;

    /**
     * 查询资产授权列表
     * @param assetAccreditQueryReq 资产授权对象
     * @return 资产授权列表信息
     */
    @PostMapping("/list")
    @Operation(summary = "查询资产授权列表", description = "查询资产授权列表")
    public Result list(@RequestBody AssetAccreditQueryReq assetAccreditQueryReq){
        List<AssetAccredit> assetAccreditList =assetAccreditService.getList(assetAccreditQueryReq);
        return Result.success(assetAccreditList);
    }

    /**
     * 添加资产授权
     * @param assetAccredit 资产授权
     * @return 添加结果
     */
    @PostMapping("/addAccredit")
    @Operation(summary = "添加资产授权", description = "添加资产授权")
    public Result addAccredit(@RequestBody AssetAccredit assetAccredit){
        assetAccreditService.save(assetAccredit);
        return Result.success();
    }

    /**
     * 删除资产授权
     * @param id 资产授权id
     * @return 删除结果
     */
    @DeleteMapping("/delAccredit/{id}")
    @Operation(summary = "删除资产授权", description = "删除资产授权")
    public Result remove(@PathVariable("id") Long id){
        assetAccreditService.removeById(id);
        return Result.success();
    }
}
