package com.muyu.source.controller;

import com.dtflys.forest.springboot.annotation.ForestScannerRegister;
import com.muyu.common.core.domain.Result;
import com.muyu.source.domain.AssetAuthorization;
import com.muyu.source.domain.rep.AssetAuthorizationRep;
import com.muyu.source.domain.req.AssetAuthorizationReq;
import com.muyu.source.service.AssetAuthorizationService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @ Tool：IntelliJ IDEA
 * @ Author：CHX
 * @ Date：2024-09-01-20:46
 * @ Version：1.0
 * @ Description：资产授权控制层
 * @author Lenovo
 */
@Log4j2
@RestController
@RequestMapping("/asset")
public class AssetAuthorizationController {

    public AssetAuthorizationController() {
        log.info("forest扫描路径：{}",
                ForestScannerRegister.getBasePackages());
    }

    @Autowired
    private AssetAuthorizationService assetAuthorizationService;

    /**
     * 查询用户id列表
     * @param assetAuthorizationReq 查询条件
     * @return Result
     */
    @PostMapping("/findUserIdList")
    public Result findUserIdList(@RequestBody AssetAuthorizationReq assetAuthorizationReq){
        List<Long> list = assetAuthorizationService.findUserIdList(assetAuthorizationReq);
        return Result.success(list);
    }

    /**
     * 添加用户权限方法
     * @param req 相应参数
     * @return 返回结果
     */
    @PostMapping("/addUserAssetAccredit")
    public Result addUserAssetAccredit(@RequestBody AssetAuthorizationReq req){
        AssetAuthorization assetImpower = new AssetAuthorization();
        assetImpower.setUserId(req.getUserId());
        assetImpower.setTableId(req.getTableId());
        assetImpower.setBasicId(req.getBasicId());
        boolean save = assetAuthorizationService.save(assetImpower);
        return save?Result.success():Result.error();
    }

    /**
     * 删除用户权限方法
     * @param req 响应参数
     * @return  返回结果
     */
    @PostMapping("/delUserAssetAccredit")
    public Result delUserAssetAccredit(@RequestBody AssetAuthorizationReq req){
        int i = assetAuthorizationService.delUserAssetAccredit(req);
        return i>0?Result.success():Result.error();
    };


    /**
     * 查询资产赋权部门信息
     * @param req 响应参数
     * @return 返回结果
     */
    @PostMapping("/findDeptIdList")
    public Result findDeptIdList(@RequestBody AssetAuthorizationReq req){
        List<Long> list = assetAuthorizationService.findDeptIdList(req);
        return Result.success(list);
    }


    /**
     * 添加用户权限方法
     * @param req 响应参数
     * @return 返回结果
     */
    @PostMapping("/addDeptAssetAccredit")
    public Result addDeptAssetAccredit(@RequestBody AssetAuthorizationReq req){
        AssetAuthorization assetImpower = new AssetAuthorization();
        assetImpower.setDeptId(req.getDeptId());
        assetImpower.setTableId(req.getTableId());
        assetImpower.setBasicId(req.getBasicId());
        boolean save = assetAuthorizationService.save(assetImpower);
        return save?Result.success():Result.error();
    }

    /**
     * 删除部门权限方法
     * @param req 响应参数
     * @return  返回结果
     */
    @PostMapping("/delDeptAssetAccredit")
    public Result delDeptAssetAccredit(@RequestBody AssetAuthorizationReq req){
        int i = assetAuthorizationService.delDeptAssetAccredit(req);
        return i>0?Result.success():Result.error();
    };

}
