package com.muyu.source.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.dtflys.forest.springboot.annotation.ForestScannerRegister;
import com.muyu.common.core.domain.Result;
import com.muyu.source.domain.Children;
import com.muyu.source.service.ChildrenService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @author Lenovo
 * @ Tool：IntelliJ IDEA
 * @ Author：CHX
 * @ Date：2024-08-21-19:05
 * @ Version：1.0
 * @ Description：数据库结构控制层
 */
@Log4j2
@RestController
@RequestMapping("/children")
@Tag(name = "数据库结构控制层", description = "进行数据库结构管理，查看等相关操作")
public class ChildrenController {
    public ChildrenController() {
        log.info("forest扫描路径:{}", ForestScannerRegister.getBasePackages());
    }

    @Autowired
    private ChildrenService childrenService;

    /**
     * 查询数据库结构
     *
     * @param id 数据库id
     * @return 数据库结构
     */
    @GetMapping("/selectChildren/{id}")
    @Operation(summary = "查询数据库结构", description = "可以根据数据源的ID查询数据库结构")
    public Result<List<Children>> selectChildren(@PathVariable("id") Integer id) {
        List<Children> children = childrenService.list(new LambdaQueryWrapper<>(){{
            eq(Children::getAssetId,id);
        }});
        return Result.success(children);
    }


}
