package com.muyu.source.controller;

import com.muyu.common.core.domain.Result;
import com.muyu.source.core.DataValue;
import com.muyu.source.domain.model.DataValueModel;
import com.muyu.source.domain.model.DataValueModels;
import com.muyu.source.service.DataValueService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.apache.commons.lang3.SystemUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.sql.SQLException;
import java.util.List;

/**
 * @author Lenovo
 * @ Tool：IntelliJ IDEA
 * @ Author：CHX
 * @ Date：2024-08-30-21:37
 * @ Version：1.0
 * @ Description：资产展示控制层
 */
@RestController
@RequestMapping("/value")
@Tag(name = "资产展示控制层", description = "进行资产展示的相关操作")
public class DataValueController {

    @Autowired
    private DataValueService dataValueService;

    /**
     * 根据基础表ID和SQL语句查询数据
     *
     * @param dataValueModel 基础表ID和sql语句
     * @return DataValue{kltv}
     */
    @PostMapping("/findTableValue")
    @Operation(summary = "根据基础表ID和SQL语句查询数据", description = "根据基础表ID和SQL语句查询数据")
    public Result findTableValue(@RequestBody DataValueModel dataValueModel) {
        // 测试耗时
        //开始执行的时间
        long begin = System.currentTimeMillis();
        List<List<DataValue>> dataValueList = dataValueService.findTableValue(dataValueModel);
        // 结束执行的时间
        long end =System.currentTimeMillis();
        //打印出执行该方法的耗时
        System.out.println("耗时："+(end-begin));
        return Result.success(dataValueList);
    }


    /**
     * 根据基础表ID和SQL语句查询数据总数
     * @param dataValueModel 基础表ID和sql语句
     * @return Integer
     */
    @PostMapping("/getTableValueTotal")
    @Operation(summary = "根据基础表ID和SQL语句查询数据总数", description = "根据基础表ID和SQL语句查询数据总数")
    public Result getTableValueTotal(@RequestBody DataValueModel dataValueModel) {
        Integer i = dataValueService.getTableValueTotal(dataValueModel);
        return Result.success(i);
    }

    /**
     * 根据基础表ID和SQL语句查询数据
     *
     * @param dataValueModel 基础表ID和sql语句
     * @return DataValue{kltv}
     */
    @PostMapping("/addTableValue")
    @Operation(summary = "根据基础表ID和SQL语句新增数据", description = "根据基础表ID和SQL语句新增数据")
    public Result addTableValue(@RequestBody DataValueModel dataValueModel) {
        Integer i = dataValueService.addTableValue(dataValueModel);
        return Result.success(i);
    }
//    /**
//     * 根据基础表ID和表名查询数据
//     *
//     * @param basicId   基础表ID
//     * @param tableName 表名
//     * @return DataValue{kltv}
//     */
//    @PostMapping("/findTableValueByTableName")
//    @Operation(summary = "根据基础表ID和SQL语句查询数据", description = "根据基础表ID和SQL语句查询数据")
//    public Result findTableValueByTableName(@RequestParam("basicId") Long basicId, @RequestParam("tableName") String tableName) {
//        List<DataValue> dataValueList = dataValueService.findTableValueByTableName(basicId, tableName);
//        return Result.success(dataValueList);
//    }

    /**
     * 给任务模块去调用
     *
     * 根据基础表ID和表名查询数据
     * @param dataValueModel 数据
     * @return DataValue[][]
     */
    @PostMapping("/findTableValueByTable")
    public Result<DataValue[][]> findTableValueByTable(@RequestBody DataValueModel dataValueModel) {
        // 测试耗时
        //开始执行的时间
        long begin = System.currentTimeMillis();
        // 执行查询的方法
        DataValue[][] dataValues =dataValueService.findTableValueByTable(dataValueModel);
        // 结束执行的时间
        long end =System.currentTimeMillis();
        //打印出执行该方法的耗时
        System.out.println("耗时："+(end-begin));
        // 返回数据
        return Result.success(dataValues);
    }
    @PostMapping("/addTableValueByType")
    public Result addTableValueByType(@RequestBody DataValueModels dataValueModel){
        //开始执行的时间
        long begin = System.currentTimeMillis();
         int i =dataValueService.addTableValueByType(dataValueModel);
        // 结束执行的时间
        long end =System.currentTimeMillis();
        //打印出执行该方法的耗时
        System.out.println("耗时："+(end-begin));
        return Result.success();
    }


    /**
     * 根据基础表ID和SQL语句查询数据
     * 给任务模块调用
     *
     * @param dataValueModel 基础表ID和sql语句
     * @return DataValue{kltv}
     */
    @PostMapping("/findTableValueBySql")
    @Operation(summary = "根据基础表ID和SQL语句查询数据", description = "根据基础表ID和SQL语句查询数据")
    public Result findTableValueBySql(@RequestBody DataValueModel dataValueModel) {
        // 测试耗时
        //开始执行的时间
        long begin = System.currentTimeMillis();
        List<List<DataValue>> dataValueList = dataValueService.findTableValueBySql(dataValueModel);
        // 结束执行的时间
        long end =System.currentTimeMillis();
        //打印出执行该方法的耗时
        System.out.println("耗时："+(end-begin));
        return Result.success(dataValueList);
    }
    /**
     * 获取表中的数据值前台调用
     * @param basicId 响应参数
     * @param tableName 响应参数
     * @return 返回结果
     */
    @PostMapping("/findTableValueList")
    @Operation(summary = "获取表中的数据值前台调用",
            description = "获取表中的数据值前台调用")
    public Result<List<DataValue>> findTableValueByTable(@RequestParam("basicId") Long basicId,@RequestParam("tableName") String tableName){
        List<DataValue> list = dataValueService.findTableValueByTable(basicId,tableName);
        return Result.success(list);
    }
    /**
     * 根据基础表ID和SQL语句查询数据总数
     * @param etlDataSqlReq 基础表ID和sql语句
     * @return Integer
     */
    @PostMapping("/getTableTotal")
    @Operation(summary = "根据基础表ID和SQL语句查询数据总数", description = "根据基础表ID和SQL语句查询数据总数")
    public Result getTableTotal(@RequestBody DataValueModel etlDataSqlReq) {
        Integer i = dataValueService.getTableTotal(etlDataSqlReq);
        return Result.success(i);
    }

    /**
     * 根据基础表ID和SQL语句查询数据
     *
     * @param etlDataSqlReq 基础表ID和sql语句
     * @return DataValue{kltv}
     */
    @PostMapping("/addTableValueBy")
    @Operation(summary = "根据基础表ID和SQL语句新增数据", description = "根据基础表ID和SQL语句新增数据")
    public Result addTableValueBy(@RequestBody DataValueModel etlDataSqlReq) {
        //开始执行的时间
        long begin = System.currentTimeMillis();
        Integer i = dataValueService.addTableValueBy(etlDataSqlReq);
        // 结束执行的时间
        long end =System.currentTimeMillis();
        //打印出执行该方法的耗时
        System.out.println("耗时："+(end-begin));
        return Result.success(i);
    }

}
