package com.muyu.source.controller;

import com.dtflys.forest.springboot.annotation.ForestScannerRegister;
import com.muyu.common.core.domain.Result;
import com.muyu.source.domain.DictData;
import com.muyu.source.service.DictDataService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author Lenovo
 * @ Tool：IntelliJ IDEA
 * @ Author：CHX
 * @ Date：2024-08-22-17:10
 * @ Version：1.0
 * @ Description：数据字典控制层
 */
@Log4j2
@RestController
@RequestMapping("/dictData")
@Tag(name = "数据字典控制层", description = "进行数据字典管理，查看等相关操作")
public class DictDataController {
    public DictDataController() {
        log.info("forest扫描路径:{}", ForestScannerRegister.getBasePackages());
    }

    @Autowired
    private DictDataService dictDataService;

    /**
     * 添加字典数据
     *
     * @param dictData 数据字典
     * @return 添加结果
     */
    @PostMapping("addDictData")
    @Operation(summary = "添加字典数据", description = "添加字典数据")
    public Result addDictData(@RequestBody DictData dictData) {
        dictDataService.save(dictData);
        return Result.success();
    }

    /**
     * 修改字典数据
     *
     * @param dictData 数据字典
     * @return 修改结果
     */
    @PostMapping("editDictData")
    @Operation(summary = "修改字典数据", description = "修改字典数据")
    public Result editDictData(@RequestBody DictData dictData) {
        dictDataService.updateById(dictData);
        return Result.success();
    }

    /**
     * 删除字典数据
     *
     * @param id 数据字典ID
     * @return 删除结果
     */
    @GetMapping("/delDictData/{id}")
    @Operation(summary = "删除字典数据", description = "根据ID删除字典数据")
    public Result delDictData(@PathVariable("id") Long id) {
        dictDataService.removeById(id);
        return Result.success();
    }

}
