package com.muyu.source.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.dtflys.forest.springboot.annotation.ForestScannerRegister;
import com.muyu.common.core.domain.Result;
import com.muyu.source.domain.DictData;
import com.muyu.source.domain.DictType;
import com.muyu.source.domain.req.DictTypeReq;
import com.muyu.source.domain.resp.DictDataResp;
import com.muyu.source.service.DictDataService;
import com.muyu.source.service.DictTypeService;
import io.swagger.v3.oas.annotations.Operation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author Lenovo
 * @ Tool：IntelliJ IDEA
 * @ Author：CHX
 * @ Date：2024-08-22-17:21
 * @ Version：1.0
 * @ Description：字典类型控制层
 */
@Log4j2
@RestController
@RequestMapping("/dictType")
public class DictTypeController {
    public DictTypeController() {
        log.info("forest扫描路径:{}", ForestScannerRegister.getBasePackages());
    }

    @Autowired
    private DictDataService dictDataService;
    @Autowired
    private DictTypeService dictTypeService;

    /**
     * 根据字典类型id查询字典数据
     *
     * @param id 字典类型id
     * @return 字典数据
     */
    @GetMapping("/getDictDataList/{id}")
    @Operation(summary = "查询字典数据", description = "根据字典类型id查询字典数据")
    public Result<List<DictDataResp>> getDictDataList(@PathVariable("id") Integer id) {
        List<DictDataResp> dictDataRespList = dictTypeService.getDictDataList(id);
        return Result.success(dictDataRespList);
    }


    /**
     * 根据对象查询字典数据列表
     *
     * @param dictTypeReq
     * @return
     */
    @PostMapping("/getDictData")
    @Operation(summary = "查询字典数据列表", description = "根据对象查询字典数据列表")
    public Result getDictData(@RequestBody DictTypeReq dictTypeReq) {
        List<DictData> dictData = dictTypeService.getDictData(dictTypeReq);
        return Result.success(dictData);
    }


    /**
     * 添加字典类型
     *
     * @param dictType 字典类型
     * @return 添加结果
     */
    @PostMapping("/addDictType")
    @Operation(summary = "添加字典类型", description = "添加字典类型")
    public Result addDictType(@RequestBody DictType dictType) {
        dictTypeService.save(dictType);
        return Result.success();
    }

    /**
     * 删除字典类型
     *
     * @param id 字典类型id
     * @return 删除结果
     */
    @GetMapping("/delDictType/{id}")
    @Operation(summary = "删除字典类型", description = "根据ID删除字典类型")
    public Result delDictType(@PathVariable("id") Long id) {
        // 删除字典类型
        dictTypeService.removeById(id);
        // 删除相对应的字典数据
        dictDataService.remove(new LambdaQueryWrapper<>() {{
            eq(DictData::getDictTypeId, id);
        }});
        return Result.success();
    }

}
