package com.muyu.source.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.dtflys.forest.annotation.NotNull;
import com.dtflys.forest.springboot.annotation.ForestScannerRegister;
import com.muyu.common.core.domain.Result;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.common.system.domain.LoginUser;
import com.muyu.common.system.domain.SysUser;
import com.muyu.source.domain.Structure;
import com.muyu.source.domain.TableInfo;
import com.muyu.source.domain.rep.AssetAuthorizationRep;
import com.muyu.source.domain.rep.TableInfoResp;
import com.muyu.source.service.AssetAuthorizationService;
import com.muyu.source.service.StructureService;
import com.muyu.source.service.TableInfoService;
import io.swagger.v3.oas.annotations.Operation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashSet;
import java.util.List;

/**
 * @author Lenovo
 */
@Log4j2
@RestController
@RequestMapping("/tableInfo")

public class TableInfoController {
    public TableInfoController() {
        log.info("forest扫描路径：{}",
                ForestScannerRegister.getBasePackages());
    }
    @Autowired
    private TableInfoService tableInfoService;
    @Autowired
    private AssetAuthorizationService assetAuthorizationService;
    @Autowired
    private StructureService structureService;

    /**
     * 查询资产信息
     * @return 查询的结果
     */
    @GetMapping("/findAsset")
    @Operation(summary = "查询数据库/表 级联查询用于数据资产授权" +
            "自己授权用" ,
            description = "查询数据库/表 级联查询用于数据资产授权" +
                    "自己授权用")
    public Result findAsset(){
        //从服务中获取TableInfo对象的列表
        return getListResult();
    }

    /**
     * 过滤级联查询方法，资产授权，获取当前用户的信息
     * 资产展示调用方法
     * @return 返回结果
     */
    @Operation(summary = "过滤级联查询方法，资产授权，获取当前用户的信息" ,
            description = "过滤级联查询方法，资产授权，获取当前用户的信息内容 赋予权限")
    @GetMapping("findAssetByTableName")
    public Result findAsserByTableName(){
        //使用安全管理工具获取当前登录的用户对象
        LoginUser loginUser = SecurityUtils.getLoginUser();
        //从登录用户对象中获取系统用户对象
        SysUser sysUser = loginUser.getSysUser();
        //从系统用户对象中获取用户ID
        Long userId = sysUser.getUserId();
        if(userId==1L){
            return getListResult();
        }
        //从系统用户对象中获取部门ID
        Long deptId = sysUser.getDeptId();

        // 调用assetAuthorizationService服务的findTableIdAndBasicIdByUserId方法
        List<AssetAuthorizationRep> idByUserId = assetAuthorizationService.findTableIdAndBasicIdByUserId(userId);

        // 调用assetAuthorizationService服务的findTableIdAndBasicIdByDeptId方法
        List<AssetAuthorizationRep> idByDeptId = assetAuthorizationService.findTableIdAndBasicIdByDeptId(deptId);


        //创建一个HashSet集合，用于存储唯一的TableInfo对象
        HashSet<TableInfo> hashSet = new HashSet<>();
        //遍历idByUserId列表，这个列表可能包含了用户授权的资产信息 findBasicIdAndTableId
        extracted(idByUserId, hashSet);

        //遍历包含用户授权信息的集合，这个集合可能是根据部门ID获取的
        extracted(idByDeptId, hashSet);
        //使用Stream API来处理列表
        List<TableInfoResp> respHashSet = hashSet.stream()
                //筛选出parentId为0的TableInfo对象
                .filter(tableInfo -> tableInfo.getParentId()==0)
                //将筛选出的TableInfo对象转换为响应对象TableInfoListResp
                .map(tableInfo -> {
                    // 将TableInfo对象转换为TableInfoListResp对象
                    TableInfoResp tableInfoResp = TableInfo.toTableInfoResp(tableInfo);
                    //递归地为每个TableInfoListResp对象设置子对象列表
                    tableInfoResp.setChildren(getChildren(tableInfo, hashSet));
                    return tableInfoResp;
                })
                //将Stream转换回List
                .toList();
        //返回成功的结果，包含处理后的列表
        return Result.success(respHashSet);
    }

    private void extracted(List<AssetAuthorizationRep> idByUserId, HashSet<TableInfo> hashSet) {
        for (AssetAuthorizationRep assetImpowerResp : idByUserId) {
            //获取资产信息中的基本ID
            Long basicId = assetImpowerResp.getBasicId();
            //获取资产信息中的表ID
            Long tableId = assetImpowerResp.getTableId();
            //如果基本原来表的ID不为空，则进行查询
            if (null != basicId) {
                //创建一个LambdaQueryWrapper，用于构建查询条件
                LambdaQueryWrapper<TableInfo> queryWrapper = new LambdaQueryWrapper<>();
                //添加查询条件，筛选出基本ID等于当前基本ID的TableInfo对象
                queryWrapper.eq(TableInfo::getBasicId, basicId);
                //调用服务方法，获取符合条件的TableInfo列表
                List<TableInfo> tableInfoBasicIdList = tableInfoService.list(queryWrapper);
                //将查询结果添加到HashSet集合中
                hashSet.addAll(tableInfoBasicIdList);
            }
            //如果当前表ID主键不为空，则进行查询
            if (null != tableId) {
                //调用服务方法，根据表ID获取TableInfo对象
                TableInfo tableInfoById = tableInfoService.getById(tableId);
                //将查询到的TableInfo对象添加到HashSet集合中
                hashSet.add(tableInfoById);
                //获取TableInfo对象的父ID
                Long parentId = tableInfoById.getParentId();
                //如果父ID不为空，则根据父ID查询父TableInfo对象
                TableInfo tableInfoByIdParentId = tableInfoService.getById(parentId);
                //将父TableInfo对象添加到HashSet集合中
                hashSet.add(tableInfoByIdParentId);
            }
        }
    }

    /**
     * 任务接口调用查询数据库/表 级联查询
     * @return 返回结果
     */
    @GetMapping("findByTableName")
    @Operation(summary = "任务接口调用查询数据库/表 级联查询" ,
            description = "任务接口调用查询数据库/表 级联查询内容")
    public Result<List<TableInfoResp>> findByTableName() {
        //从服务中获取TableInfo对象的列表
        return getListResult();
    }

    private Result<List<TableInfoResp>> getListResult() {
        List<TableInfo> list = tableInfoService.list();
        //使用Stream API来处理列表
        List<TableInfoResp> respList = list.stream()
                //筛选出parentId为0的TableInfo对象
                .filter(tableInfo -> tableInfo.getParentId()==0)
                //将筛选出的TableInfo对象转换为响应对象TableInfoListResp
                .map(tableInfo -> {
                    // 将TableInfo对象转换为TableInfoListResp对象
                    TableInfoResp tableInfoResp = TableInfo.toTableInfoResp(tableInfo);
                    //递归地为每个TableInfoListResp对象设置子对象列表
                    tableInfoResp.setChildren(getAssetChildren(tableInfo, list));
                    return tableInfoResp;
                })
                .toList();
        return Result.success(respList);
    }

    /**
     * 校验空
     * 过滤方法
     * 过滤级联查询方法，资产授权，获取当前用户的信息
     * @param tableInfo 实体类
     * @param set 结果集 去重
     * @return  返回结果
     */
    @NotNull
    private static List<TableInfoResp> getChildren(TableInfo tableInfo, HashSet<TableInfo> set) {
        //使用Stream API处理list集合
        return set.stream()
                //筛选出那些parentId等于tableInfo.getId()的TableInfo对象
                .filter(tableInfo1 -> tableInfo1.getParentId().equals(tableInfo.getId()))
                //将TableInfo对象转换为TableInfoListResp对象
                .map(
                        tableInfo2 -> TableInfo.toTableInfoResp(tableInfo2))
                .toList();
    }

    /**
     * 效验空 资产授权
     * 查询数据库/表 级联查询用于数据资产授权
     * 自己授权用 任务调用接口
     * @param tableInfo 实体类
     * @param list 结果集
     * @return 返回结果
     */
    @NotNull
    private static List<TableInfoResp> getAssetChildren(TableInfo tableInfo, List<TableInfo> list) {
        //使用Stream API处理list集合
        return list.stream()//将list转换为Stream
                //使用filter方法筛选出那些parentId等于tableInfo.getId()的TableInfo对象
                .filter(tableInfo1 -> tableInfo1.getParentId().equals(tableInfo.getId()))
                //使用map方法将筛选出的TableInfo对象转换为TableInfoListResp对象
                .map(
                        tableInfo2 -> TableInfo.toTableInfoResp(tableInfo2))
                .toList();
    }

    /**
     * 根据id查询数据集
     * @param id id
     * @return 查询的结果
     */
    @PostMapping("/findTableName")
    @Operation(summary = "根据id查询数据集" , description = "根据id查询数据集")
    public Result<TableInfo> findTableName(@RequestParam("id") Long id) {
        TableInfo byId = tableInfoService.getById(id);
        return Result.success(byId);
    }
    /**
     * 根据id查询数据集
     * @param id id
     * @return 查询的结果
     * 联调的时候任务模块调用这个
     */
    @PostMapping("/findTableById/{id}")
    @Operation(summary = "根据id查询数据集" , description = "根据id查询数据集")
    public Result<List<TableInfo>> findTableById(@PathVariable("id") Long id) {
        List<TableInfo> list =tableInfoService.findTableNameById(id);
        return Result.success(list);
    }
    @Operation(summary = "根据主键ID查询数据",description = "根据主键ID查询数据")
    @GetMapping("/getTableInfoDetails/{ids}")
    Result<List<TableInfo>> getTableInfoDetails(@PathVariable("ids") Integer[] ids){
        LambdaQueryWrapper<TableInfo> wrapper = new LambdaQueryWrapper<>();
        wrapper.in(TableInfo::getId, (Object[]) ids);
        return Result.success(tableInfoService.list(wrapper));
    }
    /**
     * 给规则引擎调用
     * 根据parentId查询数据接入
     * @return 查询的结果
     */
    @PostMapping("/selectById")
    @Operation(summary = "根据parentId查询数据接入" , description = "根据parentId查询数据接入")
    public Result<List<TableInfo>> selectById(){
        List<TableInfo> tableInfo =tableInfoService.selectById();
        return Result.success(tableInfo);
    }

    /**
     * 根据parentId向下查询出子表
     * @param id id
     * @return 返回tableInfo列表
     */
    @PostMapping("/findTableIdByParentId/{id}")
    @Operation(summary = "根据parentId向下查询出子表" , description = "根据parentId向下查询出子表")
    public Result<List<TableInfo>> findTableIdByParentId(@PathVariable("id") Long id) {
        List<Long> structureIds = tableInfoService.findTableIdByParentId(id);
        List<TableInfo> tableInfoList=tableInfoService.listByIds(structureIds);
        return Result.success(tableInfoList);
    }

    /**
     * 根据parentId向下查询出子表
     * @param parentId id
     * @return 返回表结构列表
     */
    @PostMapping("/findTableByStructure/{parentId}")
    @Operation(summary = "根据parentId向下查询出子表" , description = "根据parentId向下查询出子表")
    public Result<List<Structure>> findTableByStructure(@PathVariable("parentId") Long parentId) {
        List<Long> structureIds = structureService.findTableIdByStructureId(parentId);
        List<Structure> structureList=structureService.listByIds(structureIds);
        return Result.success(structureList);
    }
}
