package com.muyu.source.mysql;

import com.muyu.common.core.utils.SpringUtils;
import com.muyu.source.base.BaseDataAbsSource;
import com.muyu.source.core.DataType;
import com.muyu.source.core.DataValue;
import com.muyu.source.domain.DataSource;
import com.muyu.source.pool.MysqlPool;
import com.muyu.source.service.DataSourceService;
import lombok.Data;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.sql.*;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @author Lenovo
 * @ Tool：IntelliJ IDEA
 * @ Author：CHX
 * @ Date：2024-08-30-21:45
 * @ Version：1.0
 */

@Log4j2
@Component
public class MysqlDataSource extends BaseDataAbsSource {
//    DataSourceService dataSourceService = SpringUtils.getBean(DataSourceService.class);
    @Autowired
    private DataSourceService dataSourceService;

    @Override
    public DataValue getDataValue() {

        return null;
    }

    @Override
    public DataValue[] getRow() {
        // 查询数据
        MysqlQuery query = getQuery();
        // 获取到sql语句
        String sql = query.getSql();
        //获取到数据接入的ID
        Long dataSourceId = query.getDataSourceId();
        ConcurrentHashMap<Integer, DataValue> map = new ConcurrentHashMap<>();
        // 从服务中获取EtlDataScore对象，根据基本ID
        DataSource etlDataScore = dataSourceService.getById(dataSourceId);
        // 创建MySQL连接池对象，并初始化
        MysqlPool mysqlPool = new MysqlPool(etlDataScore);
        mysqlPool.init();
        // 从连接池获取数据库连接
        Connection conn = mysqlPool.getConn();

        // 初始化一个列表，用于存储数据值对象
        DataValue[] dataValues=null;

        try {
            PreparedStatement preparedStatement = conn.prepareStatement(sql);
            ResultSet resultSet = preparedStatement.executeQuery();
            ResultSetMetaData resultSetMetaData = resultSet.getMetaData();
            int columnCount = resultSetMetaData.getColumnCount();
            while (resultSet.next()){
                for (int i = 0; i < columnCount; i++) {
                    if(resultSet.isFirst()){
                        String columnTypeName = resultSetMetaData.getColumnTypeName(i);
                        DatabaseMetaData metaData = conn.getMetaData();
                        ResultSet columns = metaData.getColumns(null, null, resultSetMetaData.getTableName(i), resultSetMetaData.getColumnName(i));
                        String remarks =null;
                        while (columns.next()) {
                            remarks = columns.getString("REMARKS");
                        }
                        log.info("字段备注"+remarks);
                        DataValue dataStructure = DataValue.builder()
                                .key(resultSetMetaData.getColumnName(i))
                                .label(remarks)
                                .value(resultSet.getObject(i, DataType.getTargetClass(columnTypeName)))
                                .type(DataType.getDataType(columnTypeName))
                                .build();
                        map.put(i,dataStructure);
                        dataValues[i-1]=dataStructure;
                    }

                }
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }


        return dataValues;
    }

    @Override
    public DataValue[][] getRows() {
        // 查询数据
        MysqlQuery query = getQuery();
        //获取到sql语句
        String sql = query.getSql();
        //获取到数据接入的ID
        Long dataSourceId = query.getDataSourceId();
        //创建一个map集合，用于存储数据值对象
        ConcurrentHashMap<Integer, DataValue> map = new ConcurrentHashMap<>();
        Integer one = Math.toIntExact(query.getOne());
        Integer two = query.getTwo();
        // 从服务中获取EtlDataScore对象，根据基本ID
        DataSource etlDataScore = dataSourceService.getById(dataSourceId);
        // 创建MySQL连接池对象，并初始化
        MysqlPool mysqlPool = new MysqlPool(etlDataScore);
        // 初始化连接池
        mysqlPool.init();
        // 从连接池获取数据库连接
        Connection conn = mysqlPool.getConn();
        //new一个二维数组
        DataValue[][] dataValues =new DataValue[one][two];
        try {
            // 准备SQL查询语句
            PreparedStatement preparedStatement = conn.prepareStatement(sql);
            // 获取结果集
            ResultSet resultSet = preparedStatement.executeQuery();
            // 获取结果集的元数据
            ResultSetMetaData resultSetMetaData = resultSet.getMetaData();
            // 获取列的数量
            int columnCount = resultSetMetaData.getColumnCount();
            //定义一个变量统计行数
            int count=0;
            // 遍历结果集
            while (resultSet.next()){
                //遍历每一列
                for (int i = 0; i < columnCount; i++) {
                    //判断他是否是第一行的数据
                    if(resultSet.isFirst()){
                        //获取当前的类型名称
                        String columnTypeName = resultSetMetaData.getColumnTypeName(i);
                        // 获取数据库的元数据对象
                        DatabaseMetaData metaData = conn.getMetaData();
                        // 查询数据库元数据，获取当前列的备注信息
                        ResultSet columns = metaData.getColumns(null, null, resultSetMetaData.getTableName(i), resultSetMetaData.getColumnName(i));
                        String remarks =null;
                        // 遍历备注信息的结果集
                        while (columns.next()) {
                            // 获取当前字段的备注信息
                            remarks = columns.getString("REMARKS");
                        }
                        //构建数据值对象
                        DataValue dataStructure = DataValue.builder()
                                .key(resultSetMetaData.getColumnName(i))
                                .label(remarks)
                                .value(resultSet.getObject(i, DataType.getTargetClass(columnTypeName)))
                                .type(DataType.getDataType(columnTypeName))
                                .build();
                        // 将数据值对象放入map集合中
                        map.put(i,dataStructure);
                        // 将数据值对象放入二维数组中
                        dataValues[count][i]=dataStructure;
                    }else {
                        // 获取当前列的值，并将其转换为与数据值对象中的类型匹配的类型
                        DataValue build = DataValue.builder()
                                .key(resultSetMetaData.getColumnName(i))
                                .label(map.get(i).getLabel())
                                .value(resultSet.getObject(i, map.get(i).getType().getTargetType()))
                                .type(map.get(i).getType())
                                .build();
                        // 将转换后的数据值对象放入二维数组中
                        dataValues[count][i]=build;
                    }
                    // 行数++
                    count++;
                }
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
        // 释放数据库连接
        mysqlPool.replease(conn);
        // 关闭数据库连接
        mysqlPool.closeConn();
        return dataValues;
    }
}
