package com.muyu.source.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.source.domain.AssetAuthorization;
import com.muyu.source.domain.rep.AssetAuthorizationRep;
import com.muyu.source.domain.req.AssetAuthorizationReq;
import com.muyu.source.mapper.AssetAuthorizationMapper;
import com.muyu.source.mapper.UserMapper;
import com.muyu.source.service.AssetAuthorizationService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.muyu.source.domain.SysUser;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @ Tool：IntelliJ IDEA
 * @ Author：CHX
 * @ Date：2024-09-01-20:48
 * @ Version：1.0
 * @ Description：资产授权业务实现层
 * @author Lenovo
 */
@Service
public class AssetAuthorizationServiceImpl extends ServiceImpl<AssetAuthorizationMapper, AssetAuthorization> implements AssetAuthorizationService {

    @Autowired
    private AssetAuthorizationMapper assetAuthorizationMapper;
    @Autowired
    private UserMapper userMapper;


    /**
     * 查询授权的表ID和基本信息
     *
     * @param userId 用户ID
     * @return 授权的表ID和基本信息
     */
    @Override
    public List<AssetAuthorizationRep> findTableIdAndBasicIdByUserId(Long userId) {
        List<AssetAuthorizationRep> assetAuthorizationReps  = assetAuthorizationMapper.findTableIdAndBasicIdByUserId(userId);
        return assetAuthorizationReps;
    }
    /**
     * 根据部门ID查询授权的表ID和基本信息
     *
     * @param deptId 部门ID
     * @return 授权的表ID和基本信息
     */
    @Override
    public List<AssetAuthorizationRep> findTableIdAndBasicIdByDeptId(Long deptId) {
        List<AssetAuthorizationRep> assetAuthorizationReps  = assetAuthorizationMapper.findTableIdAndBasicIdByDeptId(deptId);
        return assetAuthorizationReps;
    }
    /**
     * 查询资产赋权用户信息
     * @param req 响应参数
     * @return 返回结果
     */
    @Override
    public List<Long> findUserIdList(AssetAuthorizationReq req) {
        List<Long> userIdList = assetAuthorizationMapper.findUserIdList(req);
        return userIdList;
    }

    /**
     * 查询资产赋权部门信息
     * @param req 响应参数
     * @return 返回结果
     */
    @Override
    public List<Long> findDeptIdList(AssetAuthorizationReq req) {
        List<Long> deptIdList = assetAuthorizationMapper.findDeptIdList(req);
        return deptIdList;
    }

    /**
     * 删除用户权限方法
     * @param req 响应参数
     * @return  返回结果
     */
    @Override
    public int delUserAssetAccredit(AssetAuthorizationReq req) {
        LambdaQueryWrapper<AssetAuthorization> queryWrapper = new LambdaQueryWrapper<>();
        if (null==req.getBasicId()){
            queryWrapper.eq(AssetAuthorization::getUserId,req.getUserId())
                    .eq(AssetAuthorization::getTableId,req.getTableId());
        }else if (null==req.getTableId()){
            queryWrapper.eq(AssetAuthorization::getUserId,req.getUserId())
                    .eq(AssetAuthorization::getBasicId,req.getBasicId());
        }

        int delete = assetAuthorizationMapper.delete(queryWrapper);
        return delete;
    }

    /**
     * 删除部门权限方法
     * @param req 响应参数
     * @return  返回结果
     */
    @Override
    public int delDeptAssetAccredit(AssetAuthorizationReq req) {
        LambdaQueryWrapper<AssetAuthorization> queryWrapper = new LambdaQueryWrapper<>();
        if (null==req.getBasicId()){
            queryWrapper.eq(AssetAuthorization::getDeptId,req.getDeptId())
                    .eq(AssetAuthorization::getTableId,req.getTableId());
        }else if (null==req.getTableId()){
            queryWrapper.eq(AssetAuthorization::getDeptId,req.getDeptId())
                    .eq(AssetAuthorization::getBasicId,req.getBasicId());
        }

        int delete = assetAuthorizationMapper.delete(queryWrapper);
        return delete;
    }

//    @Override
//    public List<Long> findUserIdList(AssetAuthorizationRep assetAuthorizationReq) {
//        QueryWrapper<SysUser> queryWrapper = new QueryWrapper<>();
//        queryWrapper.select("user_id");
//        Long tableId = assetAuthorizationReq.getTableId();
//        Long basicId = assetAuthorizationReq.getBasicId();
//        if (tableId != null) {
//            queryWrapper.eq("table_id", tableId);
//        }
//        if (basicId != null) {
//            queryWrapper.eq("basic_id", basicId);
//        }
//        return userMapper.selectList(queryWrapper)
//                .stream()
//                .map(SysUser::getUserId)
//                .collect(Collectors.toList());
//    }


}
