package com.muyu.source.service.Impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.etl.data.access.data.access.client.config.DatabaseConnectionPool;
import com.muyu.source.base.BaseQueryHandler;
import com.muyu.source.core.DataType;
import com.muyu.source.core.DataValue;
import com.muyu.source.domain.DataSource;
import com.muyu.source.domain.model.DataValueModel;
import com.muyu.source.domain.model.DataValueModels;
import com.muyu.source.mapper.DataValueMapper;
import com.muyu.source.mysql.MysqlDataSource;
import com.muyu.source.mysql.MysqlQuery;
import com.muyu.source.pool.MysqlPool;
import com.muyu.source.service.DataSourceService;
import com.muyu.source.service.DataValueService;
import com.zaxxer.hikari.HikariDataSource;
import jakarta.annotation.Resource;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.sql.*;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.*;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicInteger;

import static com.muyu.source.pool.config.BaseConfig.SELECTALL;

/**
 * @author Lenovo
 * @ Tool：IntelliJ IDEA
 * @ Author：CHX
 * @ Date：2024-08-30-21:54
 * @ Version：1.0
 * @ Description：资产展示{KLTV}业务实现层
 */
@Log4j2
@Service
public class DataValueServiceImpl extends ServiceImpl<DataValueMapper, DataValue> implements DataValueService {
    // 分页查询的批次大小
    private static final int BATCH_SIZE = 1000;
    // 每次查询的记录数
    private static final int PAGE_SIZE = 100;
    // 异步处理线程池
    private ExecutorService executorService;
    private List<CompletableFuture<Void>> futures = new ArrayList<>();
    // 标记线程池是否在关闭过程中
    private AtomicBoolean isShuttingDown = new AtomicBoolean(false);

    public DataValueServiceImpl() {
        // 初始化一个固定大小的线程池
        int corePoolSize = Runtime.getRuntime().availableProcessors();
        executorService = Executors.newFixedThreadPool(corePoolSize);
    }


    @Autowired
    private DataSourceService dataSourceService;
    @Resource
    private DatabaseConnectionPool databaseConnectionPool;

    @Override
    public List<List<DataValue>> findTableValue(DataValueModel dataValueModel) {
        ConcurrentHashMap<Integer, DataValue> map = new ConcurrentHashMap<>();
        // 从服务中获取EtlDataScore对象，根据基本ID
        DataSource etlDataScore = dataSourceService.getById(dataValueModel.getBasicId());
        // 创建MySQL连接池对象，并初始化
        MysqlPool mysqlPool = new MysqlPool(etlDataScore);
        mysqlPool.init();
        // 从连接池获取数据库连接
        Connection conn = mysqlPool.getConn();

        // 初始化一个列表，用于存储数据值对象
        List<List<DataValue>> list = new ArrayList<>();

        try {
            // 准备SQL查询语句
            PreparedStatement preparedStatement = conn.prepareStatement(dataValueModel.getSql());
            // 执行查询，获取结果集
            ResultSet resultSet = preparedStatement.executeQuery();
            // 获取结果集的元数据，用于获取列的数量和类型等信息
            ResultSetMetaData metaData = resultSet.getMetaData();
            // 获取列的数量
            int columnCount = metaData.getColumnCount();

            // 遍历结果集中的每一行数据
            while (resultSet.next()){
                ArrayList<DataValue> dataValues = new ArrayList<>();
                // 遍历每一列
                for (int i = 1; i <= columnCount; i++) {
                    if(resultSet.isFirst()){

                        // 获取当前列的类型名称
                        String columnTypeName = metaData.getColumnTypeName(i);
                        // 获取数据库的元数据对象
                        DatabaseMetaData metaDataColumns = conn.getMetaData();
                        // 查询数据库元数据，获取当前列的备注信息
                        ResultSet columns = metaDataColumns.getColumns(null, null, metaData.getTableName(i), metaData.getColumnName(i));
                        String remarks =null;
                        // 遍历备注信息的结果集
                        while (columns.next()){
                            // 获取当前字段的备注信息
                            remarks = columns.getString("REMARKS");

                            // 记录日志，显示字段的备注信息
                            log.info("字段备注:"+remarks);
                        }
                        // 构建数据值对象，包含列名、备注、值、类型等信息
                        DataValue build = DataValue.builder()
                                .key(metaData.getColumnName(i))
                                .label(remarks)
                                .value(resultSet.getObject(i, DataType.getTargetClass(columnTypeName)))
                                .type(DataType.getDataType(columnTypeName))
                                .build();
                        dataValues.add(build);
                        map.put(i, build);
                    }else {
                        DataValue build = DataValue.builder()
                                .key(metaData.getColumnName(i))// 当前列的名称
                                .label(map.get(i).getLabel())// 当前列的备注信息
                                .value(resultSet.getObject(i, map.get(i).getType().getTargetType()))// 当前列的值，类型转换
                                .type(map.get(i).getType())// 当前列的类型，转换为字符串表示
                                .build();
                        dataValues.add(build);
                    }

                }
                list.add(dataValues);
            }
        } catch (SQLException e) {
            // 如果发生SQL异常，抛出运行时异常
            throw new RuntimeException(e);
        }
        // 释放数据库连接
        mysqlPool.replease(conn);
        // 关闭数据库连接
        mysqlPool.closeConn();
        // 返回包含数据值的列表
        return list;
    }
    @Override
    public List<List<DataValue>> findTableValueBySql(DataValueModel dataValueModel) {

        //根据数据源ID获取数据源信息
        ConcurrentHashMap<Integer, DataValue> map = new ConcurrentHashMap<>();
        //初始化一个列表,用于存储数据值对象
        List<List<DataValue>> list = new ArrayList<>();
        HikariDataSource hikariDataSource = databaseConnectionPool.get(dataValueModel.getBasicId());
        Connection conn = null;
        try {
            conn = hikariDataSource.getConnection();
            //准备SQL查询
            Statement statement = conn.createStatement();
            ResultSet resultSet = statement.executeQuery(dataValueModel.getSql());
            //获取元数据
            ResultSetMetaData metaData = resultSet.getMetaData();
            //获取列的数量
            int columnCount = metaData.getColumnCount();
            //遍历每一行数据
            while (resultSet.next()) {
                List<DataValue> dataStructures = new ArrayList<>();
                // 提交任务给线程池
                for (int i = 1; i <= columnCount; i++) {
                    if (resultSet.isFirst()) {
                        String columnTypeName = metaData.getColumnTypeName(i);
                        DatabaseMetaData metaDataTwo = conn.getMetaData();
                        ResultSet columns = metaDataTwo.getColumns(null, null, metaData.getTableName(i), metaData.getColumnName(i));
                        String remarks = null;
                        while (columns.next()) {
                            remarks = columns.getString("REMARKS");
                        }
                        DataValue build = DataValue.builder()
                                .key(metaData.getColumnName(i))
                                .label(remarks)
                                .value(resultSet.getObject(i, DataType.getTargetClass(columnTypeName)))
                                .type(DataType.getDataType(columnTypeName))
                                .build();
                        dataStructures.add(build);
                        map.put(i, build);
                    } else {
                        DataValue build = DataValue.builder()
                                .key(metaData.getColumnName(i))
                                .label(map.get(i).getLabel())
                                .value(resultSet.getObject(i, map.get(i).getType().getTargetType()))
                                .type(map.get(i).getType())
                                .build();
                        dataStructures.add(build);
                    }
                }
                list.add(dataStructures);
            }
        }catch (Exception e) {
            throw new RuntimeException(e);
        }finally {
            databaseConnectionPool.returnConnection(conn);
        }
        return list;
    }

    @Override
    public List<DataValue> findTableValueByTable(Long basicId, String tableName) {

        ConcurrentHashMap<Integer, DataValue> map = new ConcurrentHashMap<>();

        ArrayList<DataValue> dataValues = new ArrayList<>();
        HikariDataSource hikariDataSource = databaseConnectionPool.get(basicId);
        Connection conn = null;
        try {
            conn = hikariDataSource.getConnection();
            // 准备SQL查询语句
            PreparedStatement preparedStatement = conn.prepareStatement(SELECTALL + tableName + " LIMIT " + PAGE_SIZE);
            // 执行查询，获取结果集
            ResultSet resultSet = preparedStatement.executeQuery();
            // 获取结果集的元数据，用于获取列的数量和类型等信息
            ResultSetMetaData metaData = resultSet.getMetaData();
            // 获取列的数量
            int columnCount = metaData.getColumnCount();

            // 遍历结果集中的每一行数据
            while (resultSet.next()) {
                // 遍历每一列
                for (int i = 1; i <= columnCount; i++) {
                    if (resultSet.isFirst()) {
                        // 获取当前列的类型名称
                        String columnTypeName = metaData.getColumnTypeName(i);
                        // 获取数据库的元数据对象
                        DatabaseMetaData metaDataColumns = conn.getMetaData();
                        // 查询数据库元数据，获取当前列的备注信息
                        ResultSet columns = metaDataColumns.getColumns(null, null, metaData.getTableName(i), metaData.getColumnName(i));
                        String remarks = null;
                        // 遍历备注信息的结果集
                        while (columns.next()) {
                            // 获取当前字段的备注信息
                            remarks = columns.getString("REMARKS");
                            // 记录日志，显示字段的备注信息
                            log.info("字段备注:" + remarks);
                        }
                        // 构建数据值对象，包含列名、备注、值、类型等信息
                        DataValue build = DataValue.builder()
                                .key(metaData.getColumnName(i))// 当前列的名称
                                .label(remarks)// 当前列的备注信息
                                .value(resultSet.getObject(i, DataType.getTargetClass(columnTypeName)))// 当前列的值，类型转换
                                .type(DataType.getDataType(columnTypeName))// 当前列的类型，转换为字符串表示
                                .build();
                        dataValues.add(build);
                        map.put(i, build);
                    } else {
                        DataValue build = DataValue.builder()
                                .key(metaData.getColumnName(i))// 当前列的名称
                                .label(map.get(i).getLabel())// 当前列的备注信息
                                .value(resultSet.getObject(i, map.get(i).getType().getTargetType()))// 当前列的值，类型转换
                                .type(map.get(i).getType())// 当前列的类型，转换为字符串表示
                                .build();
                        dataValues.add(build);
                    }
                }
            }
        } catch (SQLException e) {
            // 如果发生SQL异常，抛出运行时异常
            throw new RuntimeException(e);
        }finally {
            databaseConnectionPool.returnConnection(conn);
        }
        // 返回包含数据值的列表
        return dataValues;
    }

    @Override
    public Integer getTableTotal(DataValueModel etlDataSqlReq) {
        HikariDataSource hikariDataSource = databaseConnectionPool.get(etlDataSqlReq.getBasicId());
        Connection conn = databaseConnectionPool.getConn(hikariDataSource);
        try {
            Statement statement = conn.createStatement();
            ResultSet resultSet = statement.executeQuery(etlDataSqlReq.getSql());
            if (resultSet.next()) {
                return resultSet.getInt(1);
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }finally {
            databaseConnectionPool.returnConnection(conn);
        }
        return 0;
    }

    @Override
    public Integer addTableValueBy(DataValueModel etlDataSqlReq) {
        HikariDataSource hikariDataSource = databaseConnectionPool.get(etlDataSqlReq.getBasicId());
        Connection conn = databaseConnectionPool.getConn(hikariDataSource);
        try {
            PreparedStatement preparedStatement = conn.prepareStatement(etlDataSqlReq.getSql());
            return preparedStatement.executeUpdate();
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }finally {
            databaseConnectionPool.returnConnection(conn);
        }
    }


//    @Override
//    public List<DataValue> findTableValueByTableName(Long basicId, String tableName) {
//        List<DataValue> dataValues = new ArrayList<>();
//        DataSource dataSources = dataSourceService.getById(basicId);
//        MysqlPool mysqlPool = new MysqlPool(dataSources);
//        mysqlPool.init();
//        Connection conn = mysqlPool.getConn();
//
//        try {
//            // 准备SQL查询语句
//            String sql = SELECTALL + tableName;
//            PreparedStatement preparedStatement = conn.prepareStatement(sql);
//            // 获取结果集的元数据，用于获取列的数量和类型等信息
//            ResultSetMetaData metaData = preparedStatement.getMetaData();
//            // 获取列的数量
//            int columnCount = metaData.getColumnCount();
//
//            // 使用Map存储列的元数据，包括备注信息
//            Map<String, ColumnMeta> columnMetadata = getColumnMetadata(conn, metaData, columnCount);
//
//            // 初始化分页参数
//            int offset = 0;
//            boolean hasMorePages = true;
//
//            while (hasMorePages) {
//                // 构造带分页的SQL查询
//                String pageSql = sql + " LIMIT " + PAGE_SIZE + " OFFSET " + offset;
//                PreparedStatement pageStatement = conn.prepareStatement(pageSql);
//                ResultSet pageResultSet = pageStatement.executeQuery();
//
//                List<DataValue> pageDataValues = new ArrayList<>();
//                while (pageResultSet.next()) {
//                    List<DataValue> rowValues = new ArrayList<>();
//                    for (int i = 1; i <= columnCount; i++) {
//                        String columnName = metaData.getColumnName(i);
//                        ColumnMeta columnData = columnMetadata.get(columnName);
//                        Object value = pageResultSet.getObject(i, columnData.getType().getTargetType());
//
//                        // 构建数据值对象，包含列名、备注、值、类型等信息
//                        DataValue dataValue = DataValue.builder()
//                                .key(columnName)
//                                .label(columnData.getRemarks())
//                                .value(value)
//                                .type(columnData.getType())
//                                .build();
//                        rowValues.add(dataValue);
//                    }
//                    pageDataValues.addAll(rowValues);
//                }
//
//                // 每处理完一定数量的数据，提交一次
//                if (!pageDataValues.isEmpty()) {
//                    CompletableFuture<Void> future = processBatchAsync(pageDataValues);
//                    futures.add(future);
//                    dataValues.addAll(pageDataValues);
//                }
//
//                // 检查是否有更多页面
//                hasMorePages = pageResultSet.getFetchSize() >= PAGE_SIZE;
//                offset += PAGE_SIZE;
//            }
//
//            // 确保所有异步任务完成后再关闭线程池
//            CompletableFuture.allOf(futures.toArray(new CompletableFuture[0])).join();
//
//        } catch (SQLException e) {
//            // 如果发生SQL异常，抛出运行时异常
//            throw new RuntimeException(e);
//        } finally {
//            // 在所有异步任务完成后关闭线程池
////            shutdownExecutorService(executorService);
//
//            // 释放数据库连接
//            mysqlPool.replease(conn);
//            // 关闭数据库连接
//            mysqlPool.closeConn();
//        }
//        // 返回包含数据值的列表
//        return dataValues;
//    }
//
//    private CompletableFuture<Void> processBatchAsync(final List<DataValue> batch) {
//        // 记录任务提交的时间点
//        System.out.println("任务提交：" + LocalDateTime.now());
//
//        return CompletableFuture.runAsync(() -> {
//            try {
//                // 具体的批量处理逻辑，例如写入文件、更新数据库等
//                this.saveBatch(batch);
//            } catch (Exception e) {
//                // 异步处理中的异常处理
//                e.printStackTrace();
//            }
//            // 记录任务完成的时间点
//            System.out.println("任务完成：" + LocalDateTime.now());
//        }, executorService);
//    }
//
//    private Map<String, ColumnMeta> getColumnMetadata(Connection conn, ResultSetMetaData metaData, int columnCount) throws SQLException {
//        Map<String, ColumnMeta> metadataMap = new HashMap<>();
//        for (int i = 1; i <= columnCount; i++) {
//            String columnName = metaData.getColumnName(i);
//            String columnTypeName = metaData.getColumnTypeName(i);
//
//            // 查询数据库元数据，获取当前列的备注信息
//            ResultSet columns = conn.getMetaData().getColumns(null, null, metaData.getTableName(i), columnName);
//            String remarks = null;
//            while (columns.next()) {
//                remarks = columns.getString("REMARKS");
//            }
//
//            // 构建列元数据对象
//            ColumnMeta columnData = new ColumnMeta(columnName, remarks, DataType.findBySqlType(columnTypeName));
//            metadataMap.put(columnName, columnData);
//        }
//        return metadataMap;
//    }

    private static class ColumnMeta {
        private final String columnName;
        private final String remarks;
        private final DataType type;

        public ColumnMeta(String columnName, String remarks, DataType type) {
            this.columnName = columnName;
            this.remarks = remarks;
            this.type = type;
        }

        public String getColumnName() {
            return columnName;
        }

        public String getRemarks() {
            return remarks;
        }

        public DataType getType() {
            return type;
        }
    }

    @Override
    public Integer addTableValue(DataValueModel dataValueModel) {
        MysqlQuery mysqlQuery = new MysqlQuery();
        mysqlQuery.setDataSourceId(dataValueModel.getBasicId());
        DataSource dataSource = dataSourceService.getById(dataValueModel.getBasicId());
        MysqlPool mysqlPool = new MysqlPool(dataSource);
        mysqlPool.init();
        Connection conn = mysqlPool.getConn();
        try {
            PreparedStatement preparedStatement = conn.prepareStatement(dataValueModel.getSql());
            return preparedStatement.executeUpdate();
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public Integer getTableValueTotal(DataValueModel dataValueModel) {
        MysqlQuery mysqlQuery = new MysqlQuery();
        mysqlQuery.setDataSourceId(dataValueModel.getBasicId());
        DataSource dataSource = dataSourceService.getById(dataValueModel.getBasicId());
        MysqlPool mysqlPool = new MysqlPool(dataSource);
        mysqlPool.init();
        Connection conn = mysqlPool.getConn();
        try (Statement statement = conn.createStatement();
             ResultSet resultSet = statement.executeQuery(dataValueModel.getSql())) {
            if (resultSet.next()) {
                return resultSet.getInt(1);
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
        return 0;
    }

    /**
     * 根据基础表ID和SQL语句查询数据
     * @param dataValueModel 查询条件
     * @return 数据列表
     */
    @Override
    public DataValue[][] findTableValueByTable(DataValueModel dataValueModel) {
        MysqlQuery mysqlQuery = new MysqlQuery();
        // 设置数据接入的ID
        mysqlQuery.setDataSourceId(dataValueModel.getBasicId());
        // 设置接到的sql语句
        mysqlQuery.setSql(dataValueModel.getSql());
        // 设置二维数组的第一个参数
        mysqlQuery.setOne(dataValueModel.getOne());
        // 设置二维数组的第二个参数
        mysqlQuery.setTwo(dataValueModel.getTwo());
         BaseQueryHandler.set(mysqlQuery);
        // 创建数据源对象
        MysqlDataSource mysqlDataSource = new MysqlDataSource();
        // 获取数据
        DataValue[][] rows = mysqlDataSource.getRows();
        //返回查询的结果
        return rows;
    }

    /**
     * 根据基础表ID和SQL语句查询数据
     * @param dataValueModel 数据
     * @return List<DataValue>
     */
    @Override
    public int addTableValueByType(DataValueModels dataValueModel) {
        MysqlQuery mysqlQuery = new MysqlQuery();
        mysqlQuery.setDataSourceId(dataValueModel.getBasicId());
        DataSource dataSource = dataSourceService.getById(dataValueModel.getBasicId());
        MysqlPool mysqlPool = new MysqlPool(dataSource);
        mysqlPool.init();
        // 创建线程池
        ExecutorService executorService1 = Executors.newFixedThreadPool(8);
        // 用于记录批量插入的数量
        AtomicInteger atomicInteger = new AtomicInteger();
        // 将数据拆分成小批量
        List<DataValue[][]> batches =splitData(dataValueModel.getDataValues(),2000);
        // 获取连接
        Connection conn = mysqlPool.getConn();

        try {
            // 关闭自动提交,改为手动提交
            conn.setAutoCommit(false);
            // 提交数据
            for (DataValue[][] batch : batches) {
                // 异步提交数据
                executorService1.submit(()->{
                    try {
                        // 创建语句
                        Statement statement = conn.createStatement();
                        // 执行语句
                        statement.executeUpdate(dataValueModel.getSql());

                        atomicInteger.addAndGet(batch.length);
                    } catch (SQLException e) {
                        try {
                            //回滚事务
                            conn.rollback();
                        } catch (SQLException ex) {
                            throw new RuntimeException(ex);
                        }
                        throw new RuntimeException(e);
                    }
                });
            }
            // 等待所有任务完成
            executorService1.shutdown();
            // 等待所有任务完成
            executorService1.awaitTermination(1, TimeUnit.MINUTES);
            // 提交事务
            conn.commit();
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
            throw new RuntimeException(e);
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
            // 关闭连接
            mysqlPool.closeConn();
            // 关闭连接池
            mysqlPool.closeBaseConn();
            // 关闭线程池
            executorService1.shutdownNow();
        }
        return atomicInteger.get();
    }
    /**
     * 将数据拆分成多个小批量
     * @param dataValues 数据
     * @param batchSize 每小批量的大小
     * @return 小批量列表
     */
    private List<DataValue[][]> splitData(DataValueModels[][] dataValues, int batchSize) {
        List<DataValue[][]> batches = new ArrayList<>();
        int totalSize = dataValues.length;
        int numBatches = (int) Math.ceil((double) totalSize / batchSize);
        for (int i = 0; i < numBatches; i++) {
            int start = i * batchSize;
            int end = Math.min(start + batchSize, totalSize);
            DataValue[][] batch = new DataValue[end - start][];
            System.arraycopy(dataValues, start, batch, 0, end - start);
            batches.add(batch);
        }
        return batches;
    }


}


