package com.muyu.source.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.source.domain.DictData;
import com.muyu.source.domain.DictType;
import com.muyu.source.domain.req.DictTypeReq;
import com.muyu.source.domain.resp.DictDataResp;
import com.muyu.source.mapper.DictTypeMapper;
import com.muyu.source.service.DictDataService;
import com.muyu.source.service.DictTypeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author Lenovo
 * @ Tool：IntelliJ IDEA
 * @ Author：CHX
 * @ Date：2024-08-22-17:24
 * @ Version：1.0
 * @ Description：字典类型业务实现层
 */
@Service
public class DictTypeServiceImpl extends ServiceImpl<DictTypeMapper, DictType> implements DictTypeService {
    @Autowired
    private DictDataService dictDataService;
    /**
     * 根据字典类型id查询字典数据
     *
     * @param id 字典类型id
     * @return 字典数据
     */
    @Override
    public List<DictDataResp> getDictDataList(Integer id) {
        List<DictType> dictTypeList = list(new LambdaQueryWrapper<>() {{
            eq(DictType::getAssetId, id);
        }});

        List<DictDataResp> dictDataRespList = dictTypeList.stream().map(dictType -> {
            List<DictData> dictDataList = dictDataService.list(new LambdaQueryWrapper<>() {{
                eq(DictData::getDictTypeId, dictType.getId());
            }});
            return dictType.toDataDictResp(dictDataList);
        }).toList();

        return dictDataRespList;
    }
    /**
     * 根据对象查询字典数据列表
     *
     * @param dictTypeReq
     * @return
     */
    @Override
    public List<DictData> getDictData(DictTypeReq dictTypeReq) {
        DictType dictType = this.getOne(new LambdaQueryWrapper<>() {{
            eq(DictType::getAssetId, dictTypeReq.getAssetId());
            eq(DictType::getDictType, dictTypeReq.getDictType());
        }});
        List<DictData> dictData = dictDataService.list(new LambdaQueryWrapper<>() {{
            eq(DictData::getDictTypeId, dictType.getId());
        }});
        return dictData;
    }
}
