package com.muyu.source.service.Impl;

import cn.hutool.core.lang.Assert;
import com.alibaba.cloud.commons.lang.StringUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.source.domain.Children;
import com.muyu.source.domain.TableData;
import com.muyu.source.mapper.TableDataMapper;
import com.muyu.source.service.TableDataService;
import org.checkerframework.checker.units.qual.Acceleration;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author Lenovo
 * @ Tool：IntelliJ IDEA
 * @ Author：CHX
 * @ Date：2024-08-21-19:53
 * @ Version：1.0
 * @ Description：表结构业务实现层
 */
@Service
public class TableDataServiceImpl extends ServiceImpl<TableDataMapper, TableData> implements TableDataService {
   @Autowired
   private TableDataMapper tableDataMapper;
    /**
     * 精确查询结构
     *
     * @param id 结构主键
     * @return 结构
     */
    @Override
    public TableData selectTableDataById(Long id)
    {
        LambdaQueryWrapper<TableData> queryWrapper = new LambdaQueryWrapper<>();
        Assert.notNull(id, "id不可为空");
        queryWrapper.eq(TableData::getId, id);
        return this.getOne(queryWrapper);
    }


    /**
     * 查询结构列表
     *
     * @param tableData 结构
     * @return 结构
     */
    @Override
    public List<TableData> selectTableDataList(TableData tableData)
    {
        LambdaQueryWrapper<TableData> queryWrapper = new LambdaQueryWrapper<>();
        if (StringUtils.isNotEmpty(tableData.getName())){
            queryWrapper.like(TableData::getName, tableData.getName());
        }
        if (StringUtils.isNotEmpty(tableData.getType())){
            queryWrapper.eq(TableData::getType, tableData.getType());
        }
        return this.list(queryWrapper);
    }

    /**
     * 唯一 判断
     * @param tableData 结构
     * @return 结构
     */
    @Override
    public Boolean checkIdUnique(TableData tableData) {
        LambdaQueryWrapper<TableData> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TableData::getId, tableData.getId());
        return this.count(queryWrapper) > 0;
    }

//    @Override
//    public List<TableData> findTableDataList(Long id) {
//        LambdaQueryWrapper<TableData> lambdaQueryWrapper = new LambdaQueryWrapper<TableData>().eq(TableData::getChildrenId, id);
//        return this.list(lambdaQueryWrapper);
//    }



}
