package com.muyu.source.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.source.domain.Structure;
import com.muyu.source.domain.TableInfo;
import com.muyu.source.mapper.StructureMapper;
import com.muyu.source.mapper.TableInfoMapper;
import com.muyu.source.service.TableInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class TableServiceImpl extends ServiceImpl<TableInfoMapper, TableInfo> implements TableInfoService {
    @Autowired
    private TableInfoMapper tableInfoMapper;
    @Autowired
    private StructureMapper structureMapper;

    /**
     * 根据表名查询表信息
     * @param tableInfoInsert 查询条件
     * @return 表信息
     */
    @Override
    public TableInfo selectTableInfoByName(TableInfo tableInfoInsert) {
        LambdaQueryWrapper<TableInfo> tableInfoLambdaQueryWrapper = new LambdaQueryWrapper<>();
        tableInfoLambdaQueryWrapper.eq(TableInfo::getBasicId, tableInfoInsert.getBasicId());
        tableInfoLambdaQueryWrapper.eq(TableInfo::getTableName, tableInfoInsert.getTableName())
                .eq(TableInfo::getParentId, tableInfoInsert.getParentId());

        return  this.tableInfoMapper.selectOne(tableInfoLambdaQueryWrapper);
    }


    /**
     * 根据id查询表结构
     * @param id id
     * @return 表结构集合
     */
    @Override
    public List<Structure> selectTableInfoById(Integer id) {
        List<Structure> structureList = structureMapper.selectList(new LambdaQueryWrapper<>(Structure.class) {{
            eq(Structure::getTableId, id);
        }});

        return structureList;
    }

    @Override
    public List<TableInfo> findTableNameById(Long id) {
        LambdaQueryWrapper<TableInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TableInfo::getId, id);
        List<TableInfo> list = tableInfoMapper.selectList(queryWrapper);
        return list;
    }

    /**
     * 根据父级id查询子表id
     * @param basicId id 父级id
     * @return 子表id集合
     */
    @Override
    public List<Long> findTableIdByParentId(Long basicId) {
        return tableInfoMapper.findTableIdByParentId(basicId);
    }

    @Override
    public List<TableInfo> selectById() {
        LambdaQueryWrapper<TableInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TableInfo::getParentId, 0);
        List<TableInfo> tableInfoList = tableInfoMapper.selectList(queryWrapper);
        return tableInfoList;
    }



}
